@@ -3,7 +3,7 @@ import os
 from amino.map import Map
 
 
-class EnvVars(object):
+class EnvVars:
 
     @property
     def vars(self):
@@ -16,7 +16,7 @@ class EnvVars(object):
         return self.vars.get(name)
 
     def __setitem__(self, name, value):
-        self.vars[name] = value
+        os.environ[name] = str(value)
 
 env = EnvVars()
 
