@@ -1,14 +1,15 @@
-import os
 import re
+import sys
 import logging
 import operator
-from typing import Callable
-import sys
 from pathlib import Path
+from logging import LogRecord
+from typing import Callable, Any, Union, cast
 
 from amino.lazy import lazy
 
 import amino
+import amino.maybe
 
 VERBOSE = 15
 DDEBUG = 5
@@ -16,34 +17,57 @@ logging.addLevelName(VERBOSE, 'VERBOSE')
 logging.addLevelName(DDEBUG, 'DDEBUG')
 
 
+def seq_to_str(data: Union[str, 'amino.List', Any]) -> str:
+    return cast(amino.List, data).join_lines if isinstance(data, amino.List) else str(data)
+
+
+class LazyRecord(logging.LogRecord):
+
+    def __init__(self, name, level, pathname, lineno, msg, args, exc_info, func=None, sinfo=None,  # type: ignore
+                 **kwargs) -> None:
+        super().__init__(name, level, pathname, lineno, msg, args, exc_info, func=func, sinfo=sinfo)
+        self._data = msg
+        self._args = args
+
+    @lazy
+    def _cons_message(self) -> str:
+        data = self._data(*self._args) if callable(self._data) else self._data
+        return seq_to_str(data)
+
+    def getMessage(self) -> str:
+        return self._cons_message if self.levelname == 'DDEBUG' else super().getMessage()
+
+
 class Logger(logging.Logger):
 
     def verbose(self, message, *args, **kws):
         if self.isEnabledFor(VERBOSE):
             self._log(VERBOSE, message, args, **kws)
 
-    def ddebug(self, message, *args, **kws):
+    def ddebug(self, f: Callable[..., str], *args: Any) -> None:
         if self.isEnabledFor(DDEBUG):
-            self._log(DDEBUG, message, args, **kws)
+            self._log(DDEBUG, f, args)  # type: ignore
 
-    def caught_exception(self, when, exc, *a, **kw):
+    def caught_exception(self, when: str, exc: Exception, *a: Any, **kw: Any) -> None:
         headline = 'caught exception while {}:'.format(when)
         self.exception(headline, exc_info=(type(exc), exc, exc.__traceback__))
 
+    def makeRecord(self, name: str, level: int, fn: str, lno: int, msg: Any, args: Any, exc_info: Any, func: Any=None,
+                   extra: Any=None, sinfo: Any=None) -> LogRecord:
+        return LazyRecord(name, level, fn, lno, msg, args, exc_info, func, sinfo)
 
-logging.Logger.verbose = Logger.verbose  # type: ignore
-logging.Logger.ddebug = Logger.ddebug  # type: ignore
-logging.Logger.caught_exception = Logger.caught_exception  # type: ignore
 
-log = amino_root_logger = logging.getLogger('amino')
+def install_logger_class() -> None:
+    logging.setLoggerClass(Logger)
+
+install_logger_class()
+
+log = logging.getLogger('amino')
+amino_root_logger = log
 log.setLevel(DDEBUG)
 
 
-def install_logger_class():
-    logging.setLoggerClass(Logger)
-
-
-def amino_logger(name: str):
+def amino_logger(name: str) -> logging.Logger:
     return amino_root_logger.getChild(name)
 
 _stdout_logging_initialized = False
@@ -51,18 +75,22 @@ _stdout_logging_initialized = False
 _level_env_var = 'AMINO_LOG_LEVEL'
 
 
-def init_loglevel(handler: logging.Handler, level: int=None):
-    if level is not None:
-        handler.setLevel(level)
-    elif _level_env_var in os.environ:
-        handler.setLevel(os.environ[_level_env_var])
-    elif amino.development:
-        handler.setLevel(VERBOSE)
+def env_log_level() -> 'amino.maybe.Maybe[int]':
+    return amino.env[_level_env_var]
+
+
+def init_loglevel(handler: logging.Handler, level: int=None) -> None:
+    (
+        amino.Maybe.check(level)
+        .o(env_log_level)
+        .o(amino.Boolean(amino.development).flat_m(VERBOSE)) %
+        handler.setLevel
+    )
 
 amino_stdout_handler = logging.StreamHandler(stream=sys.stdout)
 
 
-def amino_stdout_logging(level: int=None):
+def amino_stdout_logging(level: int=None) -> None:
     global _stdout_logging_initialized
     if not _stdout_logging_initialized:
         amino_root_logger.addHandler(amino_stdout_handler)
@@ -71,12 +99,11 @@ def amino_stdout_logging(level: int=None):
 
 
 default_logfile = Path.home() / '.python' / 'log'
-_file_logging_initialized = False
 _file_fmt = ('{asctime} [{levelname} @ {name}:{funcName}:{lineno}] {message}')
 
 
 def amino_file_logging(logger: logging.Logger, level: int=logging.DEBUG,
-                       logfile=default_logfile, fmt=None):
+                       logfile: Path=default_logfile, fmt: str=None) -> None:
     logfile.parent.mkdir(exist_ok=True)
     formatter = logging.Formatter(fmt or _file_fmt, style='{')
     handler = logging.FileHandler(str(logfile))
@@ -85,11 +112,8 @@ def amino_file_logging(logger: logging.Logger, level: int=logging.DEBUG,
     init_loglevel(handler, level)
 
 
-def amino_root_file_logging(level: int=logging.DEBUG, **kw):
-    global _file_logging_initialized
-    if not _file_logging_initialized:
-        amino_file_logging(amino_root_logger, level, **kw)
-        _file_logging_initialized = True
+def amino_root_file_logging(level: int=logging.DEBUG, **kw: Any) -> None:
+    amino_file_logging(amino_root_logger, level, **kw)
 
 
 class Logging:
@@ -132,7 +156,7 @@ def logger_tree(root):
 
 def indent(strings, level, width=1):
     ws = ' ' * level * width
-    return strings.map(ws.__add__)
+    return strings.map(str).map(ws.__add__)
 
 
 def format_logger_tree(tree, fmt_logger, level=0):
