@@ -1,5 +1,4 @@
-from typing import TypeVar, Generic, Callable, Union, Any
-from typing import Tuple  # NOQA
+from typing import TypeVar, Generic, Callable, Union, Any, cast
 from functools import wraps, partial
 from operator import eq, is_not
 import inspect
@@ -7,7 +6,7 @@ import traceback
 
 from amino import boolean
 from amino.tc.base import Implicits
-from amino.func import call_by_name, I
+from amino.func import call_by_name, I, curried
 
 A = TypeVar('A')
 B = TypeVar('B')
@@ -22,7 +21,7 @@ class Maybe(Generic[A], Implicits, implicits=True):
 
     @staticmethod
     def check(value: A, checker=partial(is_not, None)):
-        return Just(value) if checker(value) else Empty()
+        return Just(value) if checker(value) else Nothing
 
     @staticmethod
     def from_call(f: Callable[..., A], *args, **kwargs):
@@ -36,7 +35,7 @@ class Maybe(Generic[A], Implicits, implicits=True):
                 stack = traceback.format_stack(frame)
                 log.exception('Maybe.from_call:')
                 log.error(''.join(stack))
-            return Empty()
+            return Nothing
 
     @staticmethod
     def typed(value: A, tpe: type):
@@ -44,25 +43,35 @@ class Maybe(Generic[A], Implicits, implicits=True):
 
     @staticmethod
     def wrap(mb: Union['Maybe[A]', None]):
-        return mb if mb is not None and isinstance(mb, Just) else Empty()
+        return mb if mb is not None and isinstance(mb, Just) else Nothing
 
     @staticmethod
     def getattr(obj, attr):
         return Maybe.check(getattr(obj, attr, None))
 
+    @staticmethod
+    @curried
+    def iff(cond: bool, a: Union[A, Callable[[], A]]) -> 'Maybe[A]':
+        return cast(Maybe, Just(call_by_name(a))) if cond else Nothing
+
+    @staticmethod
+    @curried
+    def iff_m(cond: bool, a: Union[A, Callable[[], 'Maybe[A]']]) -> 'Maybe[A]':
+        return cast(Maybe, call_by_name(a)) if cond else Nothing
+
     @property
     def _get(self) -> Union[A, None]:
         pass
 
     def cata(self, f: Callable[[A], B], b: Union[B, Callable[[], B]]) -> B:
         return (
-            f(self._get)  # type: ignore
+            f(cast(A, self._get))
             if self.is_just
             else call_by_name(b)
         )
 
     def filter(self, f: Callable[[A], B]):
-        l = lambda a: self if f(a) else Empty()
+        l = lambda a: self if f(a) else Nothing
         return self.flat_map(l)
 
     def get_or_else(self, a: Union[A, Callable[[], A]]):
@@ -81,9 +90,6 @@ class Maybe(Generic[A], Implicits, implicits=True):
     def __contains__(self, v):
         return self.contains(v)
 
-    def foreach(self, f: Callable[[A], Any]):
-        self.cata(f, None)
-
     def error(self, f: Callable[[], Any]) -> 'Maybe[A]':
         self.cata(I, f)
         return self
@@ -107,9 +113,6 @@ class Maybe(Generic[A], Implicits, implicits=True):
 
     empty = is_empty
 
-    def __nonzero__(self):
-        return self.is_just
-
     @property
     def to_list(self):
         from amino.list import List
@@ -136,14 +139,14 @@ class Maybe(Generic[A], Implicits, implicits=True):
         return self.cata(I, lambda: None)
 
 
-class Just(Maybe):
+class Just(Generic[A], Maybe[A]):
 
     __slots__ = 'x',
 
-    def __new__(tp, value: A, *args, **kwargs):
+    def __new__(tp: type, value: A, *args: Any, **kwargs: Any) -> 'Just[A]':
         return object.__new__(tp)
 
-    def __init__(self, value):
+    def __init__(self, value: A) -> None:
         self.x = value
 
     @property
@@ -165,25 +168,28 @@ class Just(Maybe):
         return hash(self._get)
 
 
-class Empty(Maybe):
+class _Nothing(Generic[A], Maybe[A]):
 
-    __object = None  # type: Empty
+    __object = None  # type: _Nothing
 
-    def __new__(tp, *args, **kwargs):
-        if Empty.__object is None:
-            Empty.__object = object.__new__(tp)
-        return Empty.__object
+    def __new__(tp: type, *args: Any, **kwargs: Any) -> '_Nothing[A]':
+        if _Nothing.__object is None:
+            _Nothing.__object = object.__new__(tp)
+        return _Nothing.__object
 
     def __str__(self):
-        return 'Empty()'
+        return 'Nothing'
 
     __repr__ = __str__
 
     def __eq__(self, other):
-        return isinstance(other, Empty)
+        return isinstance(other, _Nothing)
 
     def __hash__(self):
-        return hash('Empty')
+        return hash('Nothing')
+
+Empty = _Nothing
+Nothing: Maybe = _Nothing()
 
 
 def may(f):
@@ -200,4 +206,4 @@ def flat_may(f):
         return res if isinstance(res, Maybe) else Maybe(res)
     return wrapper
 
-__all__ = ('Maybe', 'Just', 'Empty', 'may')
+__all__ = ('Maybe', 'Just', 'may', 'Empty', 'Nothing')
