@@ -2,7 +2,7 @@ from typing import TypeVar, Callable
 
 from amino.tc.flat_map import FlatMap
 from amino.tc.applicative import Applicative
-from amino import List, L, _
+from amino import List
 
 F = TypeVar('F')
 A = TypeVar('A')
@@ -11,7 +11,7 @@ B = TypeVar('B')
 
 class Monad(FlatMap, Applicative):
 
-    def map(self, fa: F, f: Callable[[A], B]) -> F:  # type: ignore
+    def map(self, fa: F, f: Callable[[A], B]) -> F:
         return self.flat_map(fa, lambda a: self.pure(f(a)))
 
     def eff(self, fa: F, tpe: type=None):
@@ -22,12 +22,11 @@ class Monad(FlatMap, Applicative):
     def effs(self, fa: F, *args):
         from amino.eff import Eff
         types = List.wrap(args)
-        c = L(Eff)(fa, _, depth=_)
-        with_depth = lambda d, t: c(t, depth=d)
+        c = lambda a, b: Eff(fa, a, depth=b)
+        with_depth = lambda d, t: c(t, d)
         types_only = lambda: c(types, depth=len(types))
         def try_depth(h, t):
-            return (with_depth(int(h), t) if isinstance(h, int) else
-                    types_only())
+            return with_depth(int(h), t) if isinstance(h, int) else types_only()
         return types.detach_head.map2(try_depth) | types_only
 
 __all__ = ('Monad',)
